Package: m6Aboost
Type: Package
Title: m6Aboost
Version: 1.19.0
Date: 2025-06-27
Authors@R: c(person(given = "You",
                    family = "Zhou",
                    role = c("aut", "cre"),
                    email = "youzhoulearning@gmail.com",
                    comment = c(ORCID = "0000-0003-1755-9073")),
             person(given = "Kathi",
                    family = "Zarnack",
                    role = c("aut"),
                    email = "kathi.zarnack@bmls.de",
                    comment = c(ORCID = "0000-0003-3527-3378")))
Description: This package can help user to run the m6Aboost model on
        their own miCLIP2 data. The package includes functions to
        assign the read counts and get the features to run the m6Aboost
        model. The miCLIP2 data should be stored in a GRanges object.
        More details can be found in the vignette.
License: Artistic-2.0
VignetteBuilder: knitr
URL: https://github.com/ZarnackGroup/m6Aboost
BugReports: https://github.com/ZarnackGroup/m6Aboost/issues
biocViews: Sequencing, Epigenetics, Genetics, ExperimentHubSoftware
Encoding: UTF-8
RoxygenNote: 7.1.1
Imports: dplyr, rtracklayer, BSgenome, Biostrings, utils, methods,
        IRanges, ExperimentHub
Depends: S4Vectors, adabag, GenomicRanges, R (>= 4.1)
Suggests: knitr, rmarkdown, bookdown, testthat, BiocStyle,
        BSgenome.Mmusculus.UCSC.mm10
Config/pak/sysreqs: make libbz2-dev libicu-dev liblzma-dev libpng-dev
        libxml2-dev libssl-dev xz-utils zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 12:56:21 UTC
RemoteUrl: https://github.com/bioc/m6Aboost
RemoteRef: HEAD
RemoteSha: 63cb5e5f01d7d29efedc55dfc4dc5cf302e1b0aa
NeedsCompilation: no
Packaged: 2026-05-30 07:53:17 UTC; root
Author: You Zhou [aut, cre] (ORCID: <https://orcid.org/0000-0003-1755-9073>),
  Kathi Zarnack [aut] (ORCID: <https://orcid.org/0000-0003-3527-3378>)
Maintainer: You Zhou <youzhoulearning@gmail.com>
Built: R 4.6.0; ; 2026-05-30 11:26:07 UTC; unix

# Generated by roxygen2: do not edit by hand

export(CtoTAssignment)
export(m6Aboost)
export(preparingData)
export(truncationAssignment)
import(ExperimentHub)
import(GenomicRanges)
import(IRanges, except = c(collapse, slice, desc))
import(adabag)
import(dplyr, except = c(union, intersect, setdiff))
import(methods)
importFrom(BSgenome,getBSgenome)
importFrom(Biostrings,getSeq)
importFrom(S4Vectors,elementMetadata)
importFrom(S4Vectors,mcols)
importFrom(rtracklayer,import.bw)
importFrom(rtracklayer,import.gff3)
importFrom(utils,globalVariables)

# m6Aboost
Author: You Zhou, Kathi Zarnack    

---

## Introduction
N6-methyladenosine (m6A) is the most abundant internal modification in mRNA. 
It impacts many different aspects of an mRNA's life, e.g. nuclear export, 
translation, stability, etc.   

m6A individual-nucleotide resolution UV crosslinking and immunoprecipitation 
(miCLIP) and the improved **miCLIP2** are m6A antibody-based methods that allow 
the transcriptome-wide mapping of m6A sites at a single-nucleotide resolution. 
In brief, UV crosslinking of the m6A antibody to 
the modified RNA leads to truncation of reverse transcription or C-to-T 
transitions in the case of readthrough. However, due to the limited specificity 
and high cross-reactivity of the m6A antibodies, the miCLIP data comprise a 
high background signal, which hampers the reliable identification of m6A sites 
from the data. 

For accurately detecting m6A sites, we implemented an AdaBoost-based machine 
learning model (**m6Aboost**) for classifying the miCLIP2 peaks into m6A sites 
and background signals. The model was trained on high-confidence 
m6A sites that were obtained by comparing wildtype and _Mettl3_ knockout mouse 
embryonic stem cells (mESC) lacking the major methyltransferase Mettl3. For 
classification, the m6Aboost model uses a series of features, including the 
experimental miCLIP2 signal (truncation events and C-to-T transitions) as well 
as the transcript region (5'UTR, CDS, 3'UTR) and the nucleotide sequence in a 
21-nt window around the miCLIP2 peak.

The package [m6Aboost](http://bioconductor.org/packages/m6Aboost) includes the 
trained model and the functionalities to prepare the data, extract the 
required features and predict the m6A sites.

---

## Citing m6Aboost

> Körtel, Nadine#, Cornelia Rückle#, You Zhou#, Anke Busch, Peter Hoch-Kraft, 
FX Reymond Sutandy, Jacob Haase, et al. 2021. “Deep and accurate detection 
of m6A RNA modifications using miCLIP2 and m6Aboost machine learning.” 
bioRxiv. https://doi.org/10.1101/2020.12.20.423675.

---

## How to use it
Documentation (vignette and user manual) is available at the **m6Aboost's** 
Bioconductor landing page at http://bioconductor.org/packages/m6Aboost.

---

## ----style, echo=FALSE, results='asis'----------------------------------------
BiocStyle::markdown()

## ----BiocManager, eval=FALSE--------------------------------------------------
# if (!require("BiocManager"))
#     install.packages("BiocManager")
# BiocManager::install("m6Aboost")

## ----initialize, results="hide", warning=FALSE, message=FALSE-----------------
library(m6Aboost)

## ----echo=TRUE, warning=FALSE, message=FALSE----------------------------------
library(m6Aboost)
## Load the test data
testpath <- system.file("extdata", package = "m6Aboost")
test_gff3 <- file.path(testpath, "test_annotation.gff3")
test <- readRDS(file.path(testpath, "test.rds"))

test

## ----eval=TRUE, include=TRUE--------------------------------------------------
## truncationAssignment allows to assign the number of truncation events
## The input should be a GRanges object with the peaks and bigWig files
## with the truncation events (separated per strand)
truncationBw_p <- file.path(testpath, "truncation_positive.bw")
truncationBw_n <- file.path(testpath, "truncation_negative.bw")

test <- truncationAssignment(test, 
    bw_positive=truncationBw_p,
    bw_negative=truncationBw_n,
    sampleName = "WT1")

## CtoTAssignment allows to assign the number of C-to-T transitions
ctotBw_p <- file.path(testpath, "C2T_positive.bw")
ctotBw_n <- file.path(testpath, "C2T_negative.bw")
test <- CtoTAssignment(test, 
    bw_positive=ctotBw_p,
    bw_negative=ctotBw_n,
    sampleName = "CtoT_WT1")

## ----eval=FALSE, include=TRUE-------------------------------------------------
# ## E.g. for two replicates, this can be calculated as
# peak$WTmean <- (peak$WT1 + peak$WT2)/2

## -----------------------------------------------------------------------------
## Extract the features for the m6Aboost prediction
test <- preparingData(test, test_gff3, colname_reads="WTmean", 
    colname_C2T="CtoTmean")
test

## ----warning=FALSE, message=FALSE---------------------------------------------
## Note that since the test data set contains only a tiny fraction of the real 
## data, and a part of the test data belongs to the training set. Here for 
## preventing the unnecessary value change, we set the normalization to FALSE. 
out <- m6Aboost(test, "BSgenome.Mmusculus.UCSC.mm10", normalization = FALSE)
out

## ----echo=TRUE, message=FALSE-------------------------------------------------
## firstly user need to load the ExperimentHub
library(ExperimentHub)
eh <- ExperimentHub()
## "EH6021" is the retrieve record of the m6Aboost
model <-  eh[["EH6021"]]
## here shows more information about the stored model
query(eh, "m6Aboost")

## ----sessionInfo--------------------------------------------------------------
sessionInfo()

## ============================================================================
## The Methods-m6Aboost for the GRanges objects
## ----------------------------------------------------------------------------

#' @import adabag
#' @import ExperimentHub
#' @importFrom Biostrings getSeq
#' @importFrom BSgenome getBSgenome
#' @importFrom rtracklayer import.bw

## ============================================================================
## Small functions
## ----------------------------------------------------------------------------

## m6Aboost can only run agaist A sites, this step keep only the A sites
.keepA <- function(object, genome)
{
    seq <- Biostrings::getSeq(genome, object)
    object <- object[seq == "A"]
    return(object)
}

## build the data.frame for running the model
.transDF <- function(object, genome)
{
    seq <- getSeq(genome, object+10)
    df <- data.frame(seq_len(length(seq)))
    for (i in seq_len(21)) {
        k <- substr(seq,i,i)
        df[,i] <- as.factor(k)
    }
    l <- c()
    for (i in seq_len(21)) {
        l <- c(l, paste0("P",i))
    }
    colnames(df) <- l
    rownames(df) <- object$ID

    df$UTR3 <- object$UTR3
    df$UTR5 <- object$UTR5
    df$CDS <- object$CDS
    df$log2RSS <- log2(object$RSS+1)
    df$log2CtoT <- log2(object$CtoT+1)
    for (i in seq_len(24)) {
        df[,i] = as.factor(df[,i] )
    }
    return(df)
}

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
## The "m6Aboost" methods for GRanges objects.
##

#' @title m6Aboost for identify the m6A peaks from the miCILP2 data
#'
#' @description An function for calculating the relative signal strength and
#'     extracting all the features that required by the m6Aboost model for
#'     each peak.
#'
#' @author You Zhou
#'
#' @param object A GRanges object which should contains all the single
#'     nucleotide peaks of miCLIP2 experiment.
#' @param genome The name of the BSgenome that you are working with. For
#'     example "BSgenome.Mmusculus.UCSC.mm10".
#' @param normalization A logical vector which indicates whether you would like
#'     normalize the RSS and C to T reads number to the mean value of the
#'     training set of the model. This will help to reduce the false positive
#'     rate.
#'
#' @return A GRanges object with all the information that is required by the
#'     m6Aboost model.
#' @examples
#'     testpath <- system.file("extdata", package = "m6Aboost")
#'     test_gff3 <- file.path(testpath, "test_annotation.gff3")
#'     test <- readRDS(file.path(testpath, "test.rds"))
#'     test<- preparingData(test, test_gff3, colname_reads="WTmean",
#'         colname_C2T="CtoTmean")
#'
#'     ## The input of m6Aboost should be the output from preparingData function
#'     ## Please make sure that the correct BSgenome package have installed
#'     ## before running motifProfile. For example,
#'     ## library("BSgenome.Mmusculus.UCSC.mm10")
#'
#'     test <- m6Aboost(test, "BSgenome.Mmusculus.UCSC.mm10")
#' @export

m6Aboost <- function(object, genome="", normalization=TRUE)
    {
        if(!is.character(genome))
            stop("Genome should be the name of BSgenome annotation, e.g.
                BSgenome.Mmusculus.UCSC.mm10")
        if (unique(width(object)) != 1)
            stop("The input GRanges object should store the information
                of the crosslinking sites from miCLIP2 experiment. Its
                width should all equal to 1.")

        ## Keep only A sites
        genome <- BSgenome::getBSgenome(genome)
        object <- .keepA(object, genome)

        df <- .transDF(object, genome)
        ## load model
        eh <- ExperimentHub::ExperimentHub()
        model <- eh[["EH6021"]]

        df$log2SOB <- df$log2RSS
        df$log2RSS <- NULL
        if (normalization == TRUE) {
            ## The normalization factor based on the our miLCIP2 data
            m <- mean(df$log2SOB)
            df$log2SOB <- df$log2SOB/(m/0.9122623)
            c <- mean(df$log2CtoT)
            df$log2CtoT <- df$log2CtoT/(c/0.4389567)
        }

        pred_res <- adabag::predict.boosting(model, newdata = df)
        object$class <- pred_res$class
        object$prob <- pred_res$prob[,1]
        return(object)
    }

context("m6Aboost")
library(m6Aboost)

if (.Platform$OS.type != "linux") {
test_that("m6Aboost works as expected",{
    testpath <- system.file("extdata", package = "m6Aboost")
    test <- readRDS(file.path(testpath, "test.rds"))
    test_gff3 <- file.path(testpath, "test_annotation.gff3")
    test <- preparingData(test, test_gff3,
                        colname_reads="WTmean",
                        colname_C2T="CtoTmean")
    test <- m6Aboost(test, "BSgenome.Mmusculus.UCSC.mm10")

    expect_is(test, "GRanges")
}
)
}

## ============================================================================
## The Methods-truncationAssignment for the GRanges object
## ----------------------------------------------------------------------------

#' @importFrom rtracklayer import.bw
#' @importFrom S4Vectors elementMetadata

## ============================================================================
## Small functions
## ----------------------------------------------------------------------------

## Import bigWig files with miCLIP2 truncation events
.readBW <- function(file_P, file_N){
    fp <- import.bw(con = file_P)
    fn <- import.bw(con = file_N)
    strand(fp) <- "+"
    strand(fn) <- "-"
    bw <- c(fp, fn)
    return(bw)
}

## Assign scores to GRanges object
.assign_bw_to_grange <- function(grange, bw, name = ""){
    o <- findOverlaps(grange,bw)
    elementMetadata(grange)[,name] <- 0
    elementMetadata(grange)[,name][queryHits(o)] <- bw$score[subjectHits(o)]
    return(grange)
}

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
## The "truncationAssignment" method for GRanges objects.
##

#' @title truncationAssignment for assigning the truncation read counts to the
#'     GRanges object with single nucleotide peaks
#'
#' @description An function for assigning the truncation read counts from
#'     bigWig to the GRanges peaks.
#' @author You Zhou
#'
#' @param object A GRanges object which should contains all the single
#'     nucleotide peaks of miCLIP2 experiment.
#' @param bw_positive A path to the bigWig file of truncation read counts
#'     at the positive strand that output from the preprocess in the m6Aboost
#'     pipeline.
#' @param bw_negative A path to the bigWig file of truncation read counts
#'     at the negative strand that output from the preprocess in the m6Aboost
#'     pipeline.
#' @param sampleName The column name that user would like to use for indicating
#'     the name of the sample.
#'
#' @return A GRanges object with the truncation read counts.
#'
#' @examples
#' if (.Platform$OS.type != "windows") {
#'     testpath <- system.file("extdata", package = "m6Aboost")
#'     test <- readRDS(file.path(testpath, "test.rds"))
#'     truncationBw_p <- file.path(testpath, "truncation_positive.bw")
#'     truncationBw_n <- file.path(testpath, "truncation_negative.bw")
#'     test <- truncationAssignment(test, bw_positive=truncationBw_p,
#'         bw_negative=truncationBw_n, sampleName = "WT1")
#'
#' }
#'
#' @export

truncationAssignment <-
    function(object, bw_positive, bw_negative, sampleName="")
    {
        if (nchar(sampleName) == 0)
            stop("Please add a sample name to assign the bigwig files.")
        ## Import bigWig files
        bw <- .readBW(bw_positive, bw_negative)
        ## Assign scores to GRanges
        object <- .assign_bw_to_grange(object, bw, name = sampleName)
        return(object)
    }

context("truncationAssignment")
library(m6Aboost)

if (.Platform$OS.type != "windows") {
test_that("truncationAssignment works as expected",{
    testpath <- system.file("extdata", package = "m6Aboost")
    test <- readRDS(file.path(testpath, "test.rds"))
    truncationBw_p <- file.path(testpath, "truncation_positive.bw")
    truncationBw_n <- file.path(testpath, "truncation_negative.bw")
    test <- truncationAssignment(test, bw_positive=truncationBw_p,
        bw_negative=truncationBw_n, sampleName = "WT1")

    expect_is(test, "GRanges")
    expect_is(test$WT1, "numeric")
}
)
}

## ============================================================================
## The Methods-preparingData for the GRanges objects
## ----------------------------------------------------------------------------

#' @import GenomicRanges
#' @import methods
#' @rawNamespace import(IRanges, except = c(collapse, slice, desc))
#' @rawNamespace import(dplyr, except = c(union, intersect, setdiff))
#' @importFrom S4Vectors mcols
#' @importFrom utils globalVariables
#' @importFrom rtracklayer import.gff3

## ============================================================================
## Small functions
## ----------------------------------------------------------------------------

## Assign the miCLIP2 peaks to transcript regions (UTR3, UTR5, CDS)
.assignment <- function(object, UTR3, CDS, UTR5)
{
    ## Define whether the peak would overlap to at least one of the Genomic
    ## features
    object$UTR3 <- ifelse(object %over% UTR3, "YES", "NO")
    object$UTR5 <- ifelse(object %over% UTR5, "YES", "NO")
    object$CDS <- ifelse(object %over% CDS, "YES", "NO")
    return(object)
}

## To fix the global variable note
utils::globalVariables(c("elementMetadata", "queryHits" , "geneid",
    "reads", "predict.boosting", "elementMetadata<-",
    "subjectHits", "strand<-", "start<-", "end<-", "gene_id"))

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
## The "preparingData" methods for GRanges objects.
##

#' @title preparingData for the miCILP2 data
#'
#' @description A function for calculating the relative signal strength and
#'     extract the features for running the m6Aboost.
#'
#' @author You Zhou
#'
#' @param object A GRanges object which should contain all single-
#'     nucleotide peaks from the miCLIP2 experiment.
#' @param annotation A path to the annotation file. The format of the
#'     annotation file should be a gff3 file and downloaded from
#'     https://www.gencodegenes.org/
#' @param colname_reads The name of the metadata column which contains the
#'     mean value of the truncation reads number without C to T transition
#'     reads.
#' @param colname_C2T The name of the meta data column which contains the
#'     mean value of C to T transition read counts.
#'
#' @return A GRanges object with all information that is required for running
#'     the m6Aboost model.
#'
#' @examples
#'     testpath <- system.file("extdata", package = "m6Aboost")
#'     test_gff3 <- file.path(testpath, "test_annotation.gff3")
#'     test <- readRDS(file.path(testpath, "test.rds"))
#'     test<- preparingData(test, test_gff3, colname_reads="WTmean",
#'         colname_C2T="CtoTmean")
#'
#' @export

preparingData <-
    function(object, annotation, colname_reads="", colname_C2T="")
    {
        if(!isS4(object))
            stop("The input object should be a GRanges object.")
        meta_name <- names(elementMetadata(object))
        if(!colname_reads %in% meta_name |!colname_C2T %in% meta_name)
            stop("colname_reads and colname_C2T should refer to the respective
                column names in the elementMetadata of the input object.")

        ## Load annotation file
        anno <- rtracklayer::import.gff3(con = annotation)
        anno_gene <- anno[anno$type == "gene"]
        anno_gene$level <- as.numeric(anno_gene$level)
        anno_gene <-  anno_gene[order(anno_gene$level, -width(anno_gene))]

        o <- findOverlaps(object, anno_gene, select = "first")
        object$gene_id <- anno_gene$gene_id[o]
        object$gene_id[is.na(object$gene_id)] <- "NO"

        object <- as.data.frame(object)
        ## Calculate Relative Signal Strength (RSS)
        object <- object %>% group_by(gene_id) %>%
            mutate(factor=mean(get(colname_reads)))

        object <- mutate(object, RSS=get(colname_reads)/factor)
        object$factor <- NULL
        object <- makeGRangesFromDataFrame(object, keep.extra.columns = TRUE)
        ## make the column for running the m6Aboost
        object$CtoT <- elementMetadata(object)[,colname_C2T]

        UTR5 <- anno[anno$type == "five_prime_UTR"]
        UTR3 <- anno[anno$type == "three_prime_UTR"]
        CDS  <- anno[anno$type == "CDS"]

        object <- .assignment(object, UTR3, CDS, UTR5)
        return(object)
    }

context("preparingData")
library(m6Aboost)

test_that("preparingData works as expected",{
    testpath <- system.file("extdata", package = "m6Aboost")
    test <- readRDS(file.path(testpath, "test.rds"))
    test_gff3 <- file.path(testpath, "test_annotation.gff3")
    test <- preparingData(test, test_gff3,
        colname_reads="WTmean",
        colname_C2T="CtoTmean")

    expect_is(test, "GRanges")
}
)

## Here makes the metadata.csv

metadata <- data.frame(Title = "The m6Aboost machine learning model",
    Description = "The machine learning model which use for identify the m6A
        signals from miCLIP2 data set",
    BiocVersion = "3.13",
    Genome = NA,
    SourceType = "RDS",
    SourceUrl = "https://github.com/Codezy99/m6Aboost/blob/main/m6Aboost.rds",
    SourceVersion = "1",
    Species = NA,
    TaxonomyId = NA,
    Coordinate_1_based = NA,
    DataProvider = "Zarnack's lab",
    Maintainer = "You Zhou <youzhoulearning@gmail.com>",
    RDataClass = "boosting",
    DispatchClass = "Rds",
    RDataPath = "m6Aboost/m6Aboost.rds",
    Tags = "")

write.csv(metadata,file = "inst/extdata/metadata.csv", row.names=FALSE)

## ============================================================================
## The Methods-CtoTAssignment for the GRanges objects
## ----------------------------------------------------------------------------

## ============================================================================
## Small functions
## ----------------------------------------------------------------------------

## Assign bigWig files with C-to-T transitions to GRanges
.readBW_C2T <- function(file_P, file_N){
    fp <- import.bw(con = file_P)
    fn <- import.bw(con = file_N)
    strand(fp) <- "+"
    strand(fn) <- "-"
    ## shift 1nt to the A position (easy to assign the value to peaks)
    fp <- shift(fp, shift = -1)
    fn <- shift(fn, shift = 1)
    ## Combine both strands
    bw <- c(fp, fn)
    return(bw)
}

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
## The "CtoTAssignment" method for GRanges objects.
##

#' @title CtoTAssignment for assigning the truncation read counts to the
#'     GRanges object with single nucleotide peaks
#'
#' @description An function for assigning the CtoT transition read counts from
#'     bigWig.
#' @author You Zhou
#'
#' @param object A GRanges object which should contains all the single
#'     nucleotide peaks of miCLIP2 experiment.
#' @param bw_positive A path to the bigWig file of C to T transition read
#'     counts at the positive strand that output from the preprocess in the
#'     m6Aboost pipeline.
#' @param bw_negative A path to the bigWig file of C to T transition read
#'     counts at the negative strand that output from the preprocess in the
#'     m6Aboost pipeline.
#' @param sampleName The column name that user would like to use for indicating
#'     the name of the sample.
#'
#' @return A GRanges object with the truncation read counts.
#'
#' @examples
#' if (.Platform$OS.type != "windows") {
#'     testpath <- system.file("extdata", package = "m6Aboost")
#'     test <- readRDS(file.path(testpath, "test.rds"))
#'     ctotBw_p <- file.path(testpath, "C2T_positive.bw")
#'     ctotBw_n <- file.path(testpath, "C2T_negative.bw")
#'     test <- CtoTAssignment(test, bw_positive=ctotBw_p, bw_negative=ctotBw_n,
#'         sampleName = "CtoT_WT1")
#' }
#'
#' @export

CtoTAssignment <-
    function(object, bw_positive, bw_negative, sampleName="")
    {
        if (nchar(sampleName) == 0)
            stop("Please add a sample name to assign the bigWig files")

        ## Import bigWig files with C-to-T transitions
        bw <- .readBW_C2T(bw_positive, bw_negative)
        ## Assign C-to-T transitions to GRanges
        object <- .assign_bw_to_grange(object, bw, name = sampleName)
        return(object)
    }

context("CtoTAssignment")
library(m6Aboost)

if (.Platform$OS.type != "windows") {
test_that("CtoTAssignment works as expected",{
    testpath <- system.file("extdata", package = "m6Aboost")
    test <- readRDS(file.path(testpath, "test.rds"))
    ctotBw_p <- file.path(testpath, "C2T_positive.bw")
    ctotBw_n <- file.path(testpath, "C2T_negative.bw")
    test <- CtoTAssignment(test, bw_positive=ctotBw_p, bw_negative=ctotBw_n,
        sampleName = "CtoT_WT1")

    expect_is(test, "GRanges")
    expect_is(test$CtoT_WT1, "numeric")
}
)
}

---
title: "Predicting m6A sites from miCLIP2 data with m6Aboost"
author:
- name: "You Zhou"
  affiliation: 
  - Buchmann Institute for Molecular Life Sciences, Frankfurt am Main, Germany
- name: "Kathi Zarnack"
  affiliation: 
  - Buchmann Institute for Molecular Life Sciences, Frankfurt am Main, Germany
date: "`r format(Sys.time(), '%B %d, %Y')`"
bibliography: library.bib
output:
    BiocStyle::html_document:
        toc_float: true
    BiocStyle::pdf_document: default
package: m6Aboost
abstract: |
    Precisely identify the m6A sites from miCLIP data is still a challenge in 
    the epigenomics field. Here we present a workflow to determine the m6A 
    sites from the miCLIP2 data set.
vignette: |
    %\VignetteIndexEntry{m6Aboost Vignettes}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
---

```{r style, echo=FALSE, results='asis'}
BiocStyle::markdown()
```

# Introduction

N6-methyladenosine (m6A) is the most abundant internal modification in mRNA. 
It impacts many different aspects of an mRNA's life, e.g. nuclear export, 
translation, stability, etc.   

m6A individual-nucleotide resolution UV crosslinking and immunoprecipitation 
(miCLIP) and the improved **miCLIP2** are m6A antibody-based methods that allow 
the transcriptome-wide mapping of m6A sites at a single-nucleotide resolution 
[@Koertel2020][@Linder2015]. In brief, UV crosslinking of the m6A antibody to 
the modified RNA leads to truncation of reverse transcription or C-to-T 
transitions in the case of readthrough. However, due to the limited specificity 
and high cross-reactivity of the m6A antibodies, the miCLIP data comprise a 
high background signal, which hampers the reliable identification of m6A sites 
from the data. 

For accurately detecting m6A sites, we implemented an AdaBoost-based machine 
learning model (**m6Aboost**) for classifying the miCLIP2 peaks into m6A sites 
and background signals [@Koertel2020]. The model was trained on high-confidence 
m6A sites that were obtained by comparing wildtype and _Mettl3_ knockout mouse 
embryonic stem cells (mESC) lacking the major methyltransferase Mettl3. For 
classification, the m6Aboost model uses a series of features, including the 
experimental miCLIP2 signal (truncation events and C-to-T transitions) as well 
as the transcript region (5'UTR, CDS, 3'UTR) and the nucleotide sequence in a 
21-nt window around the miCLIP2 peak.

The package `r Biocpkg("m6Aboost")` includes the trained model and the 
functionalities to prepare the data, extract the required features and predict 
the m6A sites.

# Installation

The `r Biocpkg("m6Aboost")` package is available at
[https://bioconductor.org](https://bioconductor.org) and can be
installed via `BiocManager::install`:

```{r BiocManager, eval=FALSE}
if (!require("BiocManager"))
    install.packages("BiocManager")
BiocManager::install("m6Aboost")
```

A package only needs to be installed once. Load the package into an
R session with

```{r initialize, results="hide", warning=FALSE, message=FALSE}
library(m6Aboost)
```

# Pre-requisite

The workflow described herein is based on our published paper [@Koertel2020]. 
Thus we expect the user to preprocess the miCLIP2 data based on the 
preprocessing pipeline in our article [@Koertel2020]. In brief, the 
preprocessing steps include basic processing of the sequencing reads, such as 
quality filtering, barcode handling, mapping, generation of the single 
nucleotide crosslink and the C to T transition bigWig file. After the 
preprocessing, we expect the user to do the peak calling with the tool 
[PureCLIP](https://github.com/skrakau/PureCLIP) [@Krakau2017].

![**Overview of the m6Aboost preprocessing workflow**.](images/pre-workflow.png)

**Note:** If you use `m6Aboost` in published research, please cite:

> Körtel, Nadine#, Cornelia Rückle#, You Zhou#, Anke Busch, Peter Hoch-Kraft, 
FX Reymond Sutandy, Jacob Haase, et al. 2021. “Deep and accurate detection 
of m6A RNA modifications using miCLIP2 and m6Aboost machine learning.” 
bioRxiv. https://doi.org/10.1101/2020.12.20.423675.

# The m6Aboost workflow

![**Overview of the m6Aboost workflow**.](images/m6Aboost-wf.png)

# Reproducibility filtering

In order to increase the reproducibility of the prediction result, we suggest 
the user to keep the peaks which present in at least two replicates for the 
following analysis.

# Loading the test data set

The package includes a test data set which allows to test all the functions 
in the m6Aboost package. The test data set comprises a subset of the
miCLIP2 peak calling result from wildtype mESC cells [@Koertel2020], 
including 1,200 peaks (PureCLIP) in three different genes 
(ENSMUSG00000026478.14, ENSMUSG00000031960.14, ENSMUSG00000035569.17). These 
are encoded in a `GRanges` object with the following metadata:    

* The column `Reads_mean` contains the *mean value* of 
truncation events at a given peak in three replicate experiments.    
* The column `CtoT` contains the *mean value* of C-to-T transitions that are
associcated with a given peak. (Note that the C-to-T transitions at an m6A site
do occur on the C flanking the modified A in the DRACH motif.)    

In addition, the package includes the annotation file `test_gff3` which
is a subset of the full annotation in `gff3` format downloaded from 
[GENCODE](https://www.gencodegenes.org/). The test truncation and C-to-T 
transition bigWig files are a subset of the miCLIP2 signal from a wildtype 
mESC cells [@Koertel2020].

```{r echo=TRUE, warning=FALSE, message=FALSE}
library(m6Aboost)
## Load the test data
testpath <- system.file("extdata", package = "m6Aboost")
test_gff3 <- file.path(testpath, "test_annotation.gff3")
test <- readRDS(file.path(testpath, "test.rds"))

test
```

# Read count assignment

After peak calling, the user needs to assign the number of truncation events 
and C-to-T transitions that are associated with each peak. In 
`r Biocpkg("m6Aboost")`, we provide two functions, `truncationAssignment` and 
`CtoTAssignment`, to assign these counts from the imported bigWig files. Note 
that the values are first assigned separated for each replicate and then 
averaged into a mean count for the subsequent feature extraction by 
`preparingData`.

```{r eval=TRUE, include=TRUE}
## truncationAssignment allows to assign the number of truncation events
## The input should be a GRanges object with the peaks and bigWig files
## with the truncation events (separated per strand)
truncationBw_p <- file.path(testpath, "truncation_positive.bw")
truncationBw_n <- file.path(testpath, "truncation_negative.bw")

test <- truncationAssignment(test, 
    bw_positive=truncationBw_p,
    bw_negative=truncationBw_n,
    sampleName = "WT1")

## CtoTAssignment allows to assign the number of C-to-T transitions
ctotBw_p <- file.path(testpath, "C2T_positive.bw")
ctotBw_n <- file.path(testpath, "C2T_negative.bw")
test <- CtoTAssignment(test, 
    bw_positive=ctotBw_p,
    bw_negative=ctotBw_n,
    sampleName = "CtoT_WT1")
```

Next, the user needs to calculate the *mean number* of truncation events and 
C-to-T transition read counts across the replicates.

```{r eval=FALSE, include=TRUE}
## E.g. for two replicates, this can be calculated as 
peak$WTmean <- (peak$WT1 + peak$WT2)/2
```

# Extract features for the m6Aboost model

For training the m6Aboost model, we used the surrounding nucleotide sequence,
the transcript region harbouring the peak, the number of C-to-T transitions and
the relative signal strength. The function `preparingData` allows to extract
these features. Please note the function `preparingData` requires
an annotation file that was downloaded from 
[GENCODE](https://www.gencodegenes.org/).

```{r}
## Extract the features for the m6Aboost prediction
test <- preparingData(test, test_gff3, colname_reads="WTmean", 
    colname_C2T="CtoTmean")
test
```

# Prediction of m6A sites

The function `m6Aboost` performs the prediction, i.e. the classification of the 
miCLIP2 peaks into m6A sites and background. As input, the function `m6Aboost` 
uses the output object from the function `preparingData`. In addition, the user 
needs to specify the name of the `BSgenome` package associated with the species 
used for the experiment. Please note that the `BSgenome` package, which 
contains the sequence information, should be downloaded from `Bioconductor` in 
advance.

## Normalization of  numerical features

Application of the machine learning model to new data set requires that the 
data were generated by the same protocol and thus show an independent and 
identical distribution. The m6Aboost model includes two numerical features from 
the miCLIP2 data, namely relative signal strength and C-to-T transitions, which 
could systematically vary between experiments. Since in the training set, both 
features approximated a *Poisson distribution*. We recommend the user to 
normalized the values of each features in the input samples by the ratio of 
the mean for this feature between the input dataset and the training set. For 
doing this normalization, user just need to set the parameter 
`normalization = TRUE`. 

```{r warning=FALSE, message=FALSE}
## Note that since the test data set contains only a tiny fraction of the real 
## data, and a part of the test data belongs to the training set. Here for 
## preventing the unnecessary value change, we set the normalization to FALSE. 
out <- m6Aboost(test, "BSgenome.Mmusculus.UCSC.mm10", normalization = FALSE)
out
```

The function `m6Aboost` returns a GRanges object with two additional metadata 
columns:    

* The column *class* provides the classification result, i.e. whether the 
m6Aboost model predicts this site to be an m6A site or not.     
* The column *prob* includes the prediction score for a given site.    

All sites with a prediction score > 0.5 are considered as m6A sites, although a 
more stringent cutoff can be applied if needed.

## Access the machine learning model m6Aboost

The raw model **m6Aboost** is stored in `r Biocpkg("ExperimentHub")`. Users 
can access the raw model with the following code:

```{r echo=TRUE, message=FALSE}
## firstly user need to load the ExperimentHub
library(ExperimentHub)
eh <- ExperimentHub()
## "EH6021" is the retrieve record of the m6Aboost
model <-  eh[["EH6021"]]
## here shows more information about the stored model
query(eh, "m6Aboost")
```

# Session info

```{r sessionInfo}
sessionInfo()
```

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CtoTAssignment.R
\name{CtoTAssignment}
\alias{CtoTAssignment}
\title{CtoTAssignment for assigning the truncation read counts to the
    GRanges object with single nucleotide peaks}
\usage{
CtoTAssignment(object, bw_positive, bw_negative, sampleName = "")
}
\arguments{
\item{object}{A GRanges object which should contains all the single
nucleotide peaks of miCLIP2 experiment.}

\item{bw_positive}{A path to the bigWig file of C to T transition read
counts at the positive strand that output from the preprocess in the
m6Aboost pipeline.}

\item{bw_negative}{A path to the bigWig file of C to T transition read
counts at the negative strand that output from the preprocess in the
m6Aboost pipeline.}

\item{sampleName}{The column name that user would like to use for indicating
the name of the sample.}
}
\value{
A GRanges object with the truncation read counts.
}
\description{
An function for assigning the CtoT transition read counts from
    bigWig.
}
\examples{
if (.Platform$OS.type != "windows") {
    testpath <- system.file("extdata", package = "m6Aboost")
    test <- readRDS(file.path(testpath, "test.rds"))
    ctotBw_p <- file.path(testpath, "C2T_positive.bw")
    ctotBw_n <- file.path(testpath, "C2T_negative.bw")
    test <- CtoTAssignment(test, bw_positive=ctotBw_p, bw_negative=ctotBw_n,
        sampleName = "CtoT_WT1")
}

}
\author{
You Zhou
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/truncationAssignment.R
\name{truncationAssignment}
\alias{truncationAssignment}
\title{truncationAssignment for assigning the truncation read counts to the
    GRanges object with single nucleotide peaks}
\usage{
truncationAssignment(object, bw_positive, bw_negative, sampleName = "")
}
\arguments{
\item{object}{A GRanges object which should contains all the single
nucleotide peaks of miCLIP2 experiment.}

\item{bw_positive}{A path to the bigWig file of truncation read counts
at the positive strand that output from the preprocess in the m6Aboost
pipeline.}

\item{bw_negative}{A path to the bigWig file of truncation read counts
at the negative strand that output from the preprocess in the m6Aboost
pipeline.}

\item{sampleName}{The column name that user would like to use for indicating
the name of the sample.}
}
\value{
A GRanges object with the truncation read counts.
}
\description{
An function for assigning the truncation read counts from
    bigWig to the GRanges peaks.
}
\examples{
if (.Platform$OS.type != "windows") {
    testpath <- system.file("extdata", package = "m6Aboost")
    test <- readRDS(file.path(testpath, "test.rds"))
    truncationBw_p <- file.path(testpath, "truncation_positive.bw")
    truncationBw_n <- file.path(testpath, "truncation_negative.bw")
    test <- truncationAssignment(test, bw_positive=truncationBw_p,
        bw_negative=truncationBw_n, sampleName = "WT1")

}

}
\author{
You Zhou
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/m6Aboost.R
\name{m6Aboost}
\alias{m6Aboost}
\title{m6Aboost for identify the m6A peaks from the miCILP2 data}
\usage{
m6Aboost(object, genome = "", normalization = TRUE)
}
\arguments{
\item{object}{A GRanges object which should contains all the single
nucleotide peaks of miCLIP2 experiment.}

\item{genome}{The name of the BSgenome that you are working with. For
example "BSgenome.Mmusculus.UCSC.mm10".}

\item{normalization}{A logical vector which indicates whether you would like
normalize the RSS and C to T reads number to the mean value of the
training set of the model. This will help to reduce the false positive
rate.}
}
\value{
A GRanges object with all the information that is required by the
    m6Aboost model.
}
\description{
An function for calculating the relative signal strength and
    extracting all the features that required by the m6Aboost model for
    each peak.
}
\examples{
    testpath <- system.file("extdata", package = "m6Aboost")
    test_gff3 <- file.path(testpath, "test_annotation.gff3")
    test <- readRDS(file.path(testpath, "test.rds"))
    test<- preparingData(test, test_gff3, colname_reads="WTmean",
        colname_C2T="CtoTmean")

    ## The input of m6Aboost should be the output from preparingData function
    ## Please make sure that the correct BSgenome package have installed
    ## before running motifProfile. For example,
    ## library("BSgenome.Mmusculus.UCSC.mm10")

    test <- m6Aboost(test, "BSgenome.Mmusculus.UCSC.mm10")
}
\author{
You Zhou
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preparingData.R
\name{preparingData}
\alias{preparingData}
\title{preparingData for the miCILP2 data}
\usage{
preparingData(object, annotation, colname_reads = "", colname_C2T = "")
}
\arguments{
\item{object}{A GRanges object which should contain all single-
nucleotide peaks from the miCLIP2 experiment.}

\item{annotation}{A path to the annotation file. The format of the
annotation file should be a gff3 file and downloaded from
https://www.gencodegenes.org/}

\item{colname_reads}{The name of the metadata column which contains the
mean value of the truncation reads number without C to T transition
reads.}

\item{colname_C2T}{The name of the meta data column which contains the
mean value of C to T transition read counts.}
}
\value{
A GRanges object with all information that is required for running
    the m6Aboost model.
}
\description{
A function for calculating the relative signal strength and
    extract the features for running the m6Aboost.
}
\examples{
    testpath <- system.file("extdata", package = "m6Aboost")
    test_gff3 <- file.path(testpath, "test_annotation.gff3")
    test <- readRDS(file.path(testpath, "test.rds"))
    test<- preparingData(test, test_gff3, colname_reads="WTmean",
        colname_C2T="CtoTmean")

}
\author{
You Zhou
}
